/*
 * Atheros CARL9170 driver
 *
 * USB - frontend
 *
 * Copyright 2008, Johannes Berg <johannes@sipsolutions.net>
 * Copyright 2009, 2010, Christian Lamparter <chunkeey@googlemail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, see
 * http://www.gnu.org/licenses/.
 *
 * This file incorporates work covered by the following copyright and
 * permission notice:
 *    Copyright (c) 2007-2008 Atheros Communications, Inc.
 *
 *    Permission to use, copy, modify, and/or distribute this software for any
 *    purpose with or without fee is hereby granted, provided that the above
 *    copyright notice and this permission notice appear in all copies.
 *
 *    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 *    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 *    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 *    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 *    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 *    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <linux/module.h>
#include <linux/usb.h>
#include <linux/firmware.h>
#include <linux/etherdevice.h>
#include <net/mac80211.h>
#include "carl9170.h"
#include "cmd.h"
#include "hw.h"
#include "fwcmd.h"
#include "usb.h"

MODULE_AUTHOR("Johannes Berg <johannes@sipsolutions.net>");
MODULE_AUTHOR("Christian Lamparter <chunkeey@googlemail.com>");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("Atheros AR9170 802.11n USB wireless");
MODULE_FIRMWARE("carl9170-1.fw");
MODULE_FIRMWARE("carl9170-1.dsc");
MODULE_FIRMWARE("carl9170-nodsc-1.fw");
MODULE_ALIAS("ar9170usb");

static const struct carl9170_firmware_info ar9170_fws[] = {
	FILL_FW(CARL9170FW_V1, "carl9170-1.dsc", AR9170_FW_ADDR_NOUPLOAD,
		"carl9170-nodsc-1.fw", AR9170_FW_ADDR_PRAM),
	FILL_FW(CARL9170FW_V1, "carl9170-1.fw", AR9170_FW_ADDR_PRAM,
		NULL, AR9170_FW_ADDR_NOUPLOAD)
};

/*
 * Note:
 *
 * Always update our wiki's device list (located at:
 * http://wireless.kernel.org/en/users/Drivers/ar9170/devices ),
 * whenever you add a new device.
 */
static struct usb_device_id ar9170_usb_ids[] = {
	/* Atheros 9170 */
	{ USB_DEVICE(0x0cf3, 0x9170) },
	/* Atheros TG121N */
	{ USB_DEVICE(0x0cf3, 0x1001) },
	/* TP-Link TL-WN821N v2 */
	{ USB_DEVICE(0x0cf3, 0x1002), .driver_info = AR9170_WPS_BUTTON },
	/* 3Com Dual Band 802.11n USB Adapter */
	{ USB_DEVICE(0x0cf3, 0x1010) },
	/* H3C Dual Band 802.11n USB Adapter */
	{ USB_DEVICE(0x0cf3, 0x1011) },
	/* Cace Airpcap NX */
	{ USB_DEVICE(0xcace, 0x0300) },
	/* D-Link DWA 160 A1 */
	{ USB_DEVICE(0x07d1, 0x3c10) },
	/* D-Link DWA 160 A2 */
	{ USB_DEVICE(0x07d1, 0x3a09) },
	/* Netgear WNA1000 */
	{ USB_DEVICE(0x0846, 0x9040) },
	/* Netgear WNDA3100 */
	{ USB_DEVICE(0x0846, 0x9010) },
	/* Netgear WN111 v2 */
	{ USB_DEVICE(0x0846, 0x9001) },
	/* Zydas ZD1221 */
	{ USB_DEVICE(0x0ace, 0x1221) },
	/* Proxim ORiNOCO 802.11n USB */
	{ USB_DEVICE(0x1435, 0x0804) },
	/* WNC Generic 11n USB Dongle */
	{ USB_DEVICE(0x1435, 0x0326) },
	/* ZyXEL NWD271N */
	{ USB_DEVICE(0x0586, 0x3417) },
	/* Z-Com UB81 BG */
	{ USB_DEVICE(0x0cde, 0x0023) },
	/* Z-Com UB82 ABG */
	{ USB_DEVICE(0x0cde, 0x0026) },
	/* Sphairon Homelink 1202 */
	{ USB_DEVICE(0x0cde, 0x0027) },
	/* Arcadyan WN7512 */
	{ USB_DEVICE(0x083a, 0xf522) },
	/* Planex GWUS300 */
	{ USB_DEVICE(0x2019, 0x5304) },
	/* IO-Data WNGDNUS2 */
	{ USB_DEVICE(0x04bb, 0x093f) },
	/* NEC WL300NU-G */
	{ USB_DEVICE(0x0409, 0x0249) },
	/* AVM FRITZ!WLAN USB Stick N */
	{ USB_DEVICE(0x057c, 0x8401) },
	/* AVM FRITZ!WLAN USB Stick N 2.4 */
	{ USB_DEVICE(0x057c, 0x8402) },
	/* Qwest/Actiontec 802AIN Wireless N USB Network Adapter */
	{ USB_DEVICE(0x1668, 0x1200) },

	/* terminate */
	{}
};
MODULE_DEVICE_TABLE(usb, ar9170_usb_ids);

static void ar9170_usb_submit_urb(struct ar9170_usb *aru)
{
	struct urb *urb;
	unsigned long flags;
	int err;

	if (unlikely(!IS_STARTED(&aru->common)))
		return;

	spin_lock_irqsave(&aru->tx_urb_lock, flags);
	if (atomic_read(&aru->tx_submitted_urbs) >= AR9170_NUM_TX_URBS) {
		spin_unlock_irqrestore(&aru->tx_urb_lock, flags);
		return;
	}
	atomic_inc(&aru->tx_submitted_urbs);

	urb = usb_get_from_anchor(&aru->tx_pending);
	if (!urb) {
		atomic_dec(&aru->tx_submitted_urbs);
		spin_unlock_irqrestore(&aru->tx_urb_lock, flags);
		return;
	}
	spin_unlock_irqrestore(&aru->tx_urb_lock, flags);

	usb_anchor_urb(urb, &aru->tx_submitted);

	err = usb_submit_urb(urb, GFP_ATOMIC);
	if (unlikely(err)) {
		if (ar9170_nag_limiter(&aru->common))
			dev_err(&aru->udev->dev, "submit_urb failed (%d).\n",
				err);

		usb_unanchor_urb(urb);
		atomic_dec(&aru->tx_submitted_urbs);
		ar9170_tx_callback(&aru->common, urb->context);
	}

	usb_free_urb(urb);
}

static void ar9170_usb_tx_data_complete(struct urb *urb)
{
	struct sk_buff *skb = urb->context;
	struct ar9170_usb *aru = (struct ar9170_usb *)
	      usb_get_intfdata(usb_ifnum_to_if(urb->dev, 0));

	if (unlikely(!aru)) {
		dev_kfree_skb_irq(skb);
		return;
	}

	atomic_dec(&aru->tx_submitted_urbs);

	ar9170_usb_submit_urb(aru);

	if (aru->common.tx_stream)
		skb_pull(skb, sizeof(struct ar9170_stream));

	ar9170_tx_callback(&aru->common, skb);
}

static void ar9170_usb_tx_cmd_complete(struct urb *urb)
{
}

static void ar9170_usb_rx_irq_complete(struct urb *urb)
{
	struct ar9170_usb *aru = urb->context;

	if (unlikely(!aru))
		goto free;

	switch (urb->status) {
	/* everything is fine */
	case 0:
		break;

	/* disconnect */
	case -ENOENT:
	case -ECONNRESET:
	case -ENODEV:
	case -ESHUTDOWN:
		goto free;

	default:
		goto resubmit;
	}

	ar9170_handle_command_response(&aru->common, urb->transfer_buffer,
				       urb->actual_length);

resubmit:
	usb_anchor_urb(urb, &aru->rx_submitted);
	if (usb_submit_urb(urb, GFP_ATOMIC)) {
		usb_unanchor_urb(urb);
		goto free;
	}

	return;

free:
	usb_buffer_free(aru->udev, AR9170_USB_EP_CTRL_MAX,
			urb->transfer_buffer, urb->transfer_dma);
}

static void ar9170_usb_rx_complete(struct urb *urb)
{
	struct ar9170_usb *aru = (struct ar9170_usb *) urb->context;
	int err;

	if (unlikely(WARN_ON_ONCE(!aru)))
		goto out;

	switch (urb->status) {
	/* everything is fine */
	case 0:
		break;

	/* disconnect */
	case -ENOENT:
	case -ECONNRESET:
	case -ENODEV:
	case -ESHUTDOWN:
		goto out;

	default:
		goto resubmit;
	}

	ar9170_rx(&aru->common, urb->transfer_buffer, urb->actual_length);

resubmit:

	usb_anchor_urb(urb, &aru->rx_submitted);
	err = usb_submit_urb(urb, GFP_ATOMIC);
	if (unlikely(err)) {
		usb_unanchor_urb(urb);
		goto out;
	}

out:
	return;
}

static struct urb *ar9170_usb_alloc_rx_urb(struct ar9170_usb *aru, gfp_t gfp)
{
	struct urb *urb;
	void *buf;

	buf = kmalloc(aru->rx_size, gfp);
	if (!buf)
		return NULL;

	urb = usb_alloc_urb(0, gfp);
	if (!urb) {
		kfree(buf);
		return NULL;
	}

	usb_fill_bulk_urb(urb, aru->udev,
			  usb_rcvbulkpipe(aru->udev, AR9170_USB_EP_RX),
			  buf, aru->rx_size, ar9170_usb_rx_complete,
			  aru);

	urb->transfer_flags |= URB_FREE_BUFFER;

	return urb;
}

static int ar9170_usb_alloc_rx_irq_urb(struct ar9170_usb *aru)
{
	struct urb *urb = NULL;
	void *ibuf;
	int err = -ENOMEM;

	/* initialize interrupt endpoint */
	urb = usb_alloc_urb(0, GFP_KERNEL);
	if (!urb)
		goto out;

	ibuf = usb_buffer_alloc(aru->udev, AR9170_USB_EP_CTRL_MAX,
				GFP_KERNEL, &urb->transfer_dma);
	if (!ibuf)
		goto out;

	usb_fill_int_urb(urb, aru->udev,
			 usb_rcvintpipe(aru->udev, AR9170_USB_EP_IRQ), ibuf,
			 AR9170_USB_EP_CTRL_MAX, ar9170_usb_rx_irq_complete,
			 aru, 1);
	urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;

	usb_anchor_urb(urb, &aru->rx_submitted);
	err = usb_submit_urb(urb, GFP_KERNEL);
	if (err) {
		usb_unanchor_urb(urb);
		usb_buffer_free(aru->udev, AR9170_USB_EP_CTRL_MAX,
				urb->transfer_buffer, urb->transfer_dma);
	}

out:
	usb_free_urb(urb);
	return err;
}

static int ar9170_usb_init_rx_bulk_urbs(struct ar9170_usb *aru)
{
	struct urb *urb;
	int i;
	int err = -EINVAL;

	for (i = 0; i < AR9170_NUM_RX_URBS; i++) {
		urb = ar9170_usb_alloc_rx_urb(aru, GFP_KERNEL);
		if (!urb) {
			err = -ENOMEM;
			goto err_out;
		}

		usb_anchor_urb(urb, &aru->rx_submitted);
		err = usb_submit_urb(urb, GFP_KERNEL);
		if (err) {
			usb_unanchor_urb(urb);
			usb_free_urb(urb);
			goto err_out;
		}
		usb_free_urb(urb);
	}

	/* the device now waiting for a firmware. */
	aru->common.state = AR9170_IDLE;
	return 0;

err_out:

	usb_kill_anchored_urbs(&aru->rx_submitted);
	return err;
}

int ar9170_usb_flush(struct ar9170 *ar)
{
	struct ar9170_usb *aru = (void *) ar;
	struct urb *urb;
	int ret, err = 0;

	if (IS_STARTED(ar))
		aru->common.state = AR9170_IDLE;

	usb_wait_anchor_empty_timeout(&aru->tx_pending,
					    msecs_to_jiffies(800));
	while ((urb = usb_get_from_anchor(&aru->tx_pending))) {
		ar9170_tx_callback(&aru->common, (void *) urb->context);
		usb_free_urb(urb);
	}

	/* lets wait a while until the tx - queues are dried out */
	ret = usb_wait_anchor_empty_timeout(&aru->tx_submitted,
					    msecs_to_jiffies(100));
	if (ret == 0)
		err = -ETIMEDOUT;

	usb_kill_anchored_urbs(&aru->tx_submitted);

	if (IS_ACCEPTING_CMD(ar))
		aru->common.state = AR9170_STARTED;

	return err;
}

static void ar9170_usb_cancel_urbs(struct ar9170_usb *aru)
{
	int err;

	aru->common.state = AR9170_UNKNOWN_STATE;

	err = ar9170_usb_flush(&aru->common);
	if (err)
		dev_err(&aru->udev->dev, "stuck tx urbs!\n");

	usb_poison_anchored_urbs(&aru->tx_submitted);
	usb_poison_anchored_urbs(&aru->rx_submitted);
}

int ar9170_usb_exec_cmd(struct ar9170 *ar, enum ar9170_cmd_oids cmd,
			unsigned int plen, void *payload,
			unsigned int outlen, void *out)
{
	struct ar9170_usb *aru = (void *) ar;
	struct urb *urb = NULL;
	unsigned long flags;
	int err = -ENOMEM;

	might_sleep();

	if (unlikely(!IS_ACCEPTING_CMD(ar)))
		return -EPERM;

	if (WARN_ON(plen > CARL9170_MAX_CMD_LEN - 4))
		return -EINVAL;

	urb = usb_alloc_urb(0, GFP_ATOMIC);
	if (unlikely(!urb))
		goto err_free;

	ar->cmdbuf[0] = cpu_to_le32(plen);
	ar->cmdbuf[0] |= cpu_to_le32(cmd << 8);
	/* writing multiple regs fills this buffer already */
	if (plen && payload != (u8 *)(&ar->cmdbuf[1]))
		memcpy(&ar->cmdbuf[1], payload, plen);

	spin_lock_irqsave(&ar->cmdlock, flags);
	aru->readbuf = (u8 *)out;
	aru->readlen = outlen;
	spin_unlock_irqrestore(&ar->cmdlock, flags);

	usb_fill_int_urb(urb, aru->udev,
			 usb_sndintpipe(aru->udev, AR9170_USB_EP_CMD),
			 ar->cmdbuf, plen + 4,
			 ar9170_usb_tx_cmd_complete, NULL, 1);

	usb_anchor_urb(urb, &aru->tx_submitted);
	err = usb_submit_urb(urb, GFP_ATOMIC);
	if (unlikely(err)) {
		usb_unanchor_urb(urb);
		usb_free_urb(urb);
		goto err_unbuf;
	}
	usb_free_urb(urb);

	err = wait_for_completion_timeout(&aru->cmd_wait, HZ);
	if (err == 0) {
		err = -ETIMEDOUT;
		goto err_unbuf;
	}

	if (aru->readlen != outlen) {
		err = -EMSGSIZE;
		goto err_unbuf;
	}

	return 0;

err_unbuf:
	/* Maybe the device was removed in the second we were waiting? */
	if (IS_STARTED(ar)) {
		dev_err(&aru->udev->dev, "no command feedback "
					 "received (%d).\n", err);

		/* provide some maybe useful debug information */
		print_hex_dump_bytes("ar9170 cmd: ", DUMP_PREFIX_NONE,
				     ar->cmdbuf, plen + 4);

		ar9170_restart(ar);
	}

	/* invalidate to avoid completing the next prematurely */
	spin_lock_irqsave(&ar->cmdlock, flags);
	aru->readbuf = NULL;
	aru->readlen = 0;
	spin_unlock_irqrestore(&ar->cmdlock, flags);

err_free:

	return err;
}

int ar9170_usb_tx(struct ar9170 *ar, struct sk_buff *skb)
{
	struct ar9170_usb *aru = (struct ar9170_usb *) ar;
	struct urb *urb;
	struct ar9170_stream *tx_stream;

	if (unlikely(!IS_STARTED(ar))) {
		/* Seriously, what were you drink... err... thinking!? */
		return -EPERM;
	}

	urb = usb_alloc_urb(0, GFP_ATOMIC);
	if (unlikely(!urb))
		return -ENOMEM;

	if (ar->tx_stream) {
		tx_stream = (void *) skb_push(skb, sizeof(*tx_stream));

		tx_stream->length = cpu_to_le16(skb->len);
		tx_stream->tag = cpu_to_le16(AR9170_TX_STREAM_TAG);
	}

	usb_fill_bulk_urb(urb, aru->udev,
			  usb_sndbulkpipe(aru->udev, AR9170_USB_EP_TX),
			  skb->data, skb->len,
			  ar9170_usb_tx_data_complete, skb);
	urb->transfer_flags |= URB_ZERO_PACKET;

	usb_anchor_urb(urb, &aru->tx_pending);

	usb_free_urb(urb);

	ar9170_usb_submit_urb(aru);
	return 0;
}

void ar9170_usb_callback_cmd(struct ar9170 *ar, u32 len, void *buffer)
{
	struct ar9170_usb *aru = (void *) ar;
	unsigned long flags;

	if (unlikely(!buffer))
		return;

	/*
	 * Some commands (e.g: AR9170_CMD_FREQUENCY) have a variable response
	 * length and we cannot predict the correct length in advance.
	 * So we only check if we provided enough space for the data.
	 */
	if (unlikely(aru->readlen != (len - 4))) {
		dev_warn(&aru->udev->dev, "received invalid command response "
					  "got %d bytes, instead of %d bytes\n",
			 len - 4, aru->readlen);
		print_hex_dump_bytes("ar9170 invalid resp: ",
				     DUMP_PREFIX_OFFSET, buffer, len);
		/*
		 * Do not complete, then the command times out,
		 * and we get a stack trace from there.
		 */
		return;
	}

	spin_lock_irqsave(&ar->cmdlock, flags);
	if (aru->readbuf) {
		if (len >= 4)
			memcpy(aru->readbuf, buffer + 4, len - 4);
		aru->readbuf = NULL;
	}
	complete(&aru->cmd_wait);
	spin_unlock_irqrestore(&ar->cmdlock, flags);
}

static int ar9170_usb_upload(struct ar9170_usb *aru, const void *data,
			     size_t len, u32 addr, bool complete)
{
	int transfer, err;
	u8 *buf = kmalloc(4096, GFP_KERNEL);

	if (!buf)
		return -ENOMEM;

	while (len) {
		transfer = min_t(int, len, 4096);
		memcpy(buf, data, transfer);

		err = usb_control_msg(aru->udev, usb_sndctrlpipe(aru->udev, 0),
				      0x30 /* FW DL */, 0x40 | USB_DIR_OUT,
				      addr >> 8, 0, buf, transfer, 100);

		if (err < 0) {
			kfree(buf);
			return err;
		}

		len -= transfer;
		data += transfer;
		addr += transfer;
	}
	kfree(buf);

	err = 0;

	if (complete) {
		err = usb_control_msg(aru->udev, usb_sndctrlpipe(aru->udev, 0),
				      0x31 /* FW DL COMPLETE */,
				      0x40 | USB_DIR_OUT, 0, 0, NULL, 0, 200);

		/*
		 * the boot command can sometimes fail, but
		 * we know everything is OK, when the firmware
		 * responds to the ECHO command
		 *
		 * Wait! Here's a race, the device could already have
		 * vanished.
		 *
		 * err = ar9170_echo_test(&aru->common, 0x4a110123);
		 */
	}

	return err;
}

static void ar9170_release_firmwares(struct ar9170_usb *aru)
{
	unsigned int i;

	for (i = 0; i < aru->fw_files; i++) {
		release_firmware(aru->firmware[i]);
		aru->firmware[i] = NULL;
	}
	aru->fw_files = 0;
}

static const u8 usb_magic[4] = { USB_MAGIC };

static int ar9170_usb_check_firmware(struct ar9170_usb *aru)
{
	const struct carl9170fw_usb_desc_v1 *usb_desc;
	int err;

	err = ar9170_parse_firmware(&aru->common, aru->firmware[0],
				    aru->firmware[aru->fw_files - 1]);
	if (err) {
		dev_err(&aru->udev->dev, "failed to parse firmware (%d).\n",
			err);
		goto fail;
	}

	usb_desc = ar9170_fw_find_desc(&aru->common, usb_magic,
		sizeof(*usb_desc), CARL9170FW_USB_DESC_CUR_VER);
	if (!usb_desc) {
		dev_err(&aru->udev->dev, "not an compatible usb firmware.\n");
		err = -EINVAL;
		goto fail;
	}

#define SUPP(feat) (carl9170fw_supports(usb_desc->usb_feature_set, feat))

	if (!SUPP(CARL9170FW_USB_DUMMY_FEATURE)) {
		dev_err(&aru->udev->dev, "usb firmware does not pass "
			"sanity checks\n");
		err = -ECANCELED;
		goto fail;
	}

	if (!SUPP(AR9170FW_USB_INIT_FIRMWARE)) {
		dev_err(&aru->udev->dev, "usb firmware does not provide "
					 "mandatory interfaces.\n");
		err = -EINVAL;
		goto fail;
	}

	if (usb_desc->tx_descs < 16 ||
	    le16_to_cpu(usb_desc->tx_frag_len) < 64 ||
	    le16_to_cpu(usb_desc->tx_frag_len) > 512 ||
	    le16_to_cpu(usb_desc->rx_max_frame_len) < AR9170_MAX_RX_BUFFER_SIZE ||
	    le16_to_cpu(usb_desc->rx_max_frame_len) > 32768) {
		dev_err(&aru->udev->dev, "usb firmware shows obvious signs of "
					 "malicious tampering.\n");
		err = -EINVAL;
		goto fail;
	}

	if (SUPP(CARL9170FW_USB_MINIBOOT))
		aru->fw_offset = le16_to_cpu(usb_desc->miniboot_size);
	else
		aru->fw_offset = 0;

	if (SUPP(AR9170FW_USB_DOWN_STREAM)) {
		aru->common.hw->extra_tx_headroom +=
			sizeof(struct ar9170_stream);
		aru->common.tx_stream = true;
	}

	if (SUPP(AR9170FW_USB_UP_STREAM))
		aru->common.rx_stream = true;

	if (SUPP(CARL9170FW_USB_WATCHDOG))
		aru->common.usb_watchdog = true;

	if (ilog2(le32_to_cpu(usb_desc->usb_feature_set)) >=
		__CARL9170FW_USB_FEATURE_NUM) {
		dev_warn(&aru->udev->dev, "driver does not support all "
			 "firmware features.\n");
	}

	aru->rx_size = le16_to_cpu(usb_desc->rx_max_frame_len);
	aru->common.mem_blocks = usb_desc->tx_descs;
	atomic_set(&aru->common.mem_free_blocks, usb_desc->tx_descs);
	aru->common.mem_block_size = le16_to_cpu(usb_desc->tx_frag_len);

	return 0;

fail:

	ar9170_release_firmwares(aru);
	return err;

#undef SUPP
}

void ar9170_usb_stop(struct ar9170 *ar)
{
	struct ar9170_usb *aru = (void *) ar;
	int ret;

	if (IS_ACCEPTING_CMD(ar))
		aru->common.state = AR9170_STOPPED;

	ret = ar9170_usb_flush(ar);
	if (ret)
		dev_err(&aru->udev->dev, "kill pending tx urbs.\n");

	usb_poison_anchored_urbs(&aru->tx_submitted);

	/*
	 * Note:
	 * So far we freed all tx urbs, but we won't dare to touch any rx urbs.
	 * Else we would end up with a unresponsive device...
	 */
}

int ar9170_usb_open(struct ar9170 *ar)
{
	struct ar9170_usb *aru = (void *) ar;

	usb_unpoison_anchored_urbs(&aru->tx_submitted);

	aru->common.state = AR9170_IDLE;
	return 0;
}

static int ar9170_usb_load_firmware(struct ar9170_usb *aru)
{
	unsigned int i;
	int err = 0;

	for (i = 0; i < aru->fw_files; i++) {
		const void *fw_data;
		size_t fw_size;
		u32 addr;
		bool boot = false;

		fw_data = aru->firmware[i]->data;
		fw_size = aru->firmware[i]->size;
		addr = aru->fw_info->files[i].address;

		if (addr == AR9170_FW_ADDR_NOUPLOAD)
			continue;

		/*
		 * boot the firmware if this was the last file entry
		 * or the next one is empty.
		 */
		if (i == (AR9170_FW_MAX_FILES - 1) ||
		    !aru->fw_info->files[i + 1].name)
			boot = true;

		/* this removes the miniboot image */
		if (boot) {
			fw_data += aru->fw_offset;
			fw_size -= aru->fw_offset;
		}

		err = ar9170_usb_upload(aru, fw_data, fw_size, addr, boot);

		/* let it settle */
		msleep(100);

		if (err) {
			ar9170_usb_cancel_urbs(aru);
			dev_err(&aru->udev->dev, "device is in a bad state. "
						 "please reconnect it!\n");

			/* force user invention, by disabling the device */
			usb_driver_set_configuration(aru->udev, -1);
			return err;
		}
	}

	return err;
}

int ar9170_usb_restart(struct ar9170 *ar, bool needs_usb_reset)
{
	struct ar9170_usb *aru = (void *) ar;
	int err = 0;

	if (aru->intf->condition != USB_INTERFACE_BOUND)
		return 0;

	if (!needs_usb_reset)
		err = ar9170_reboot(&aru->common);

	ar9170_usb_stop(ar);
	aru->common.state = AR9170_UNKNOWN_STATE;

	if (!err && !needs_usb_reset) {
		ar9170_usb_open(ar);
		err = ar9170_usb_load_firmware(aru);
	}

	if (needs_usb_reset || err) {
		/*
		 * This is the last resort to get the device going againg
		 * without any *user replugging action*.
		 *
		 * But there is a catch: usb_reset acts like a physical
		 * *reconnect*. And the mac80211 state will be lost,
		 * Therefore the userspace must step in an initiate the
		 * whole reconnect sequence.
		 */
		usb_queue_reset_device(aru->intf);

		err = -ENODEV;
	}

	return err;
}

static int ar9170_usb_init_device(struct ar9170_usb *aru)
{
	int err;

	err = ar9170_usb_alloc_rx_irq_urb(aru);
	if (err)
		goto err_out;

	err = ar9170_usb_init_rx_bulk_urbs(aru);
	if (err)
		goto err_unrx;

	err = ar9170_usb_load_firmware(aru);
	if (err)
		goto err_unrx;

	return 0;

err_unrx:
	ar9170_usb_cancel_urbs(aru);

err_out:
	return err;
}

static int ar9170_usb_request_firmware(struct ar9170_usb *aru);

static int ar9170_usb_probe(struct usb_interface *intf,
			const struct usb_device_id *id)
{
	struct ar9170_usb *aru;
	struct ar9170 *ar;
	struct usb_device *udev;
	int err;

	err = usb_reset_device(interface_to_usbdev(intf));
	if (err)
		return err;

	aru = ar9170_alloc(sizeof(*aru));
	if (IS_ERR(aru))
		return PTR_ERR(aru);

	udev = interface_to_usbdev(intf);
	usb_get_dev(udev);
	aru->udev = udev;
	aru->common.pdev = &udev->dev;
	aru->intf = intf;
	aru->quirks = id->driver_info;
	ar = &aru->common;

	usb_set_intfdata(intf, aru);
	SET_IEEE80211_DEV(ar->hw, &intf->dev);

	init_usb_anchor(&aru->rx_submitted);
	init_usb_anchor(&aru->tx_pending);
	init_usb_anchor(&aru->tx_submitted);
	init_completion(&aru->cmd_wait);
	init_completion(&aru->fw_load_wait);
	spin_lock_init(&aru->tx_urb_lock);

	atomic_set(&aru->tx_submitted_urbs, 0);

#ifdef CONFIG_CARL9170_WPS_BUTTON
	aru->common.has_wps_button = !!(aru->quirks & AR9170_WPS_BUTTON);
#endif /* CONFIG_CARL9170_WPS_BUTTON */

	usb_get_dev(aru->udev);
	return ar9170_usb_request_firmware(aru);
}

static void ar9170_usb_firmware_failed(struct ar9170_usb *aru)
{
	struct device *parent = aru->udev->dev.parent;

	complete(&aru->fw_load_wait);

	/* unbind anything failed */
	if (parent)
		device_lock(parent);

	device_release_driver(&aru->udev->dev);
	if (parent)
		device_unlock(parent);

	usb_put_dev(aru->udev);
}

static void ar9170_usb_firmware_finish(struct ar9170_usb *aru)
{
	int err;

	aru->common.firmware_type = aru->fw_info->type;

	err = ar9170_usb_check_firmware(aru);
	if (err)
		goto err_freefw;

	err = ar9170_usb_init_device(aru);
	if (err)
		goto err_freefw;

	err = ar9170_usb_open(&aru->common);
	if (err)
		goto err_unrx;

	err = ar9170_register(&aru->common);

	ar9170_usb_stop(&aru->common);
	if (err)
		goto err_unrx;

#ifdef CONFIG_CARL9170_DEBUGFS
	ar9170_register_debugfs(aru);
#endif /* CONFIG_CARL9170_DEBUGFS */

	complete(&aru->fw_load_wait);
	usb_put_dev(aru->udev);
	return;

err_unrx:
	ar9170_usb_cancel_urbs(aru);

err_freefw:
	ar9170_release_firmwares(aru);
	ar9170_usb_firmware_failed(aru);
}

static void ar9170_usb_firmware_step2(const struct firmware *fw, void *context)
{
	struct ar9170_usb *aru = context;
	int err;

	if (fw) {
		aru->firmware[aru->fw_files] = fw;
		aru->fw_files++;

		/* try to load next file, if there is any... */
		if (aru->fw_files < AR9170_FW_MAX_FILES &&
		    ar9170_fws[aru->fw_load].files[aru->fw_files].name)
			goto request_fw;

		if (aru->fw_files > 1)
			aru->common.separate_desc_file = true;

		aru->fw_info = &ar9170_fws[aru->fw_load];
		ar9170_usb_firmware_finish(aru);
		return;
	}

	ar9170_release_firmwares(aru);

	/* try next firmware --- if possible */
	aru->fw_load++;

	if (aru->fw_load > FIRMWARE_NUM) {
		dev_err(&aru->udev->dev, "firmware(s) not found.\n");
		goto err_failed;
	}

request_fw:
	err = ar9170_usb_request_firmware(aru);
	return;

err_failed:
	ar9170_usb_firmware_failed(aru);
}

static int ar9170_usb_request_firmware(struct ar9170_usb *aru)
{
	return request_firmware_nowait(THIS_MODULE, 1,
		ar9170_fws[aru->fw_load].files[aru->fw_files].name,
		&aru->udev->dev, GFP_KERNEL, aru, ar9170_usb_firmware_step2);
}

static void ar9170_usb_disconnect(struct usb_interface *intf)
{
	struct ar9170_usb *aru = usb_get_intfdata(intf);

	if (!aru)
		return;

	wait_for_completion(&aru->fw_load_wait);

#ifdef CONFIG_CARL9170_DEBUGFS
	ar9170_unregister_debugfs(aru);
#endif /* CONFIG_CARL9170_DEBUGFS */

	aru->common.state = AR9170_IDLE;
	ar9170_reboot(&aru->common);

	aru->common.state = AR9170_UNKNOWN_STATE;
	ar9170_unregister(&aru->common);
	ar9170_usb_cancel_urbs(aru);

	usb_put_dev(aru->udev);
	usb_set_intfdata(intf, NULL);

	ar9170_release_firmwares(aru);
	ieee80211_free_hw(aru->common.hw);
}

#ifdef CONFIG_PM
static int ar9170_usb_suspend(struct usb_interface *intf,
			      pm_message_t message)
{
	struct ar9170_usb *aru = usb_get_intfdata(intf);

	if (!aru)
		return -ENODEV;

	ar9170_usb_cancel_urbs(aru);

	/*
	 * firmware automatically reboots for usb suspend.
	 */

	return 0;
}

static int ar9170_usb_resume(struct usb_interface *intf)
{
	struct ar9170_usb *aru = usb_get_intfdata(intf);
	int err;

	if (!aru)
		return -ENODEV;

	usb_unpoison_anchored_urbs(&aru->rx_submitted);
	usb_unpoison_anchored_urbs(&aru->tx_submitted);

	err = ar9170_usb_init_device(aru);
	if (err)
		goto err_unrx;

	err = ar9170_usb_open(&aru->common);
	if (err)
		goto err_unrx;

	return 0;

err_unrx:
	aru->common.state = AR9170_UNKNOWN_STATE;
	ar9170_usb_cancel_urbs(aru);

	return err;
}
#endif /* CONFIG_PM */

static struct usb_driver carl9170_driver = {
	.name = KBUILD_MODNAME,
	.probe = ar9170_usb_probe,
	.disconnect = ar9170_usb_disconnect,
	.id_table = ar9170_usb_ids,
	.soft_unbind = 1,
#ifdef CONFIG_PM
	.suspend = ar9170_usb_suspend,
	.resume = ar9170_usb_resume,
#endif /* CONFIG_PM */
};

static int __init carl9170_usb_init(void)
{
	return usb_register(&carl9170_driver);
}

static void __exit carl9170_usb_exit(void)
{
	usb_deregister(&carl9170_driver);
}

module_init(carl9170_usb_init);
module_exit(carl9170_usb_exit);
